@@ -52,48 +52,40 @@
  * @w: The Width output
  * @h: The Height output
  *
- * Returns: Success or failure
+ * Returns: 
  */
-gboolean
-rsvg_css_parse_vbox (const char * vbox, double * x, double * y,
-					 double * w, double * h)
+RsvgViewBox
+rsvg_css_parse_vbox (const char * vbox)
 {
+	RsvgViewBox vb;
 	gdouble * list;
 	guint list_len;
+	vb.active = FALSE;
 
 	list = rsvg_css_parse_number_list(vbox, &list_len);
 
 	if(!(list && list_len))
-		return FALSE;
+		return vb;
 	else if(list_len != 4) {
 		g_free(list);
-		return FALSE;
+		return vb;
 	} 
 	else {
-		*x = list[0];
-		*y = list[1];
-		*w = list[2];
-		*h = list[3];
+		vb.x = list[0];
+		vb.y = list[1];
+		vb.w = list[2];
+		vb.h = list[3];
+		vb.active = TRUE;
 
 		g_free(list);
-		return TRUE;
+		return vb;
 	}
 }
 
 
-/**
- * rsvg_css_parse_length: Parse CSS2 length to a pixel value.
- * @str: Original string.
- * @pixels_per_inch: Pixels per inch
- * @fixed: Where to store boolean value of whether length is fixed.
- *
- * Parses a CSS2 length into a pixel value.
- *
- * Returns: returns the length.
- **/
-double
-rsvg_css_parse_length (const char *str, gint *in, 
-					   gint *percent, gint *em, gint *ex)
+static double
+rsvg_css_parse_raw_length (const char *str, gint *in, 
+						   gint *percent, gint *em, gint *ex)
 {
 	double length = 0.0;
 	char *p = NULL;
@@ -152,13 +144,13 @@ rsvg_css_parse_length (const char *str, gint *in,
 }
 
 RsvgLength
-_rsvg_css_parse_length_struct(const char *str)
+_rsvg_css_parse_length(const char *str)
 {
 	RsvgLength out;
 	gint percent, em, ex, in;
 	percent = em = ex = in = FALSE;
 	
-	out.length = rsvg_css_parse_length (str, &in, &percent, &em, &ex);
+	out.length = rsvg_css_parse_raw_length (str, &in, &percent, &em, &ex);
 	if (percent)
 		out.factor = 'p';
 	else if (em)
@@ -173,7 +165,7 @@ _rsvg_css_parse_length_struct(const char *str)
 }
 
 double
-_rsvg_css_normalize_length_struct(const RsvgLength * in, RsvgDrawingCtx * ctx, 
+_rsvg_css_normalize_length(const RsvgLength * in, RsvgDrawingCtx * ctx, 
 								  char dir)
 {
 	if (in->factor == '\0')
@@ -184,23 +176,32 @@ _rsvg_css_normalize_length_struct(const RsvgLength * in, RsvgDrawingCtx * ctx,
 				return in->length * ctx->vb.w;
 			if (dir == 'v')
 				return in->length * ctx->vb.h;
+			if (dir == 'o')
+				return in->length * rsvg_viewport_percentage(ctx->vb.w, ctx->vb.h);
+		}
+	else if (in->factor == 'm' || in->factor == 'x')
+		{
+			double font = _rsvg_css_hand_normalize_length(&rsvg_state_current(ctx)->font_size,
+																 ctx->dpi_y, ctx->vb.h, 1);	
+			if (in->factor == 'm')
+				return in->length * font;
+			else
+				return in->length * font / 2.;
 		}
-	else if (in->factor == 'm')
-		return in->length * rsvg_state_current(ctx)->font_size;
-	else if (in->factor == 'x')
-		return in->length * rsvg_state_current(ctx)->font_size / 2.;
 	else if (in->factor == 'i')
 		{
 			if (dir == 'h')
 				return in->length * ctx->dpi_x;
 			if (dir == 'v')
 				return in->length * ctx->dpi_y;
+			if (dir == 'o')
+				return in->length * rsvg_viewport_percentage(ctx->dpi_x, ctx->dpi_y);
 		}
 	return 0;
 }
 
 double
-_rsvg_css_hand_normalize_length_struct(const RsvgLength * in, gdouble pixels_per_inch,
+_rsvg_css_hand_normalize_length(const RsvgLength * in, gdouble pixels_per_inch,
 									   gdouble width_or_height, gdouble font_size)
 {
 	if (in->factor == '\0')
@@ -217,37 +218,6 @@ _rsvg_css_hand_normalize_length_struct(const RsvgLength * in, gdouble pixels_per
 	return 0;
 }
 
-/**
- * rsvg_css_parse_normalized_length: Parse CSS2 length to a pixel value.
- * @str: Original string.
- * @pixels_per_inch: Pixels per inch
- * @normalize_to: Bounding box's width or height, as appropriate, or 0
- *
- * Parses a CSS2 length into a pixel value.
- *
- * Returns: returns the length.
- */
-double
-rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
-								 gdouble width_or_height, gdouble font_size)
-{
-	double length;
-	gint percent, em, ex, in;
-	percent = em = ex = in = FALSE;
-	
-	length = rsvg_css_parse_length (str, &in, &percent, &em, &ex);
-	if (percent)
-		return length * width_or_height;
-	else if (em)
-		return length * font_size;
-	else if (ex)
-		return (length * font_size) / 2.;
-	else if (in)
-		return length * pixels_per_inch;
-	else
-		return length;
-}
-
 gboolean
 rsvg_css_param_match (const char *str, const char *param_name)
 {
